/* Copyright (c) (2019) Apple Inc. All rights reserved.
 *
 * corecrypto is licensed under Apple Inc.’s Internal Use License Agreement (which
 * is contained in the License.txt file distributed with corecrypto) and only to 
 * people who accept that license. IMPORTANT:  Any license rights granted to you by 
 * Apple Inc. (if any) are limited to internal use within your organization only on 
 * devices and computers you own or control, for the sole purpose of verifying the 
 * security characteristics and correct functioning of the Apple Software.  You may 
 * not, directly or indirectly, redistribute the Apple Software or any portions thereof.
*/


#include <stdint.h>
#include <corecrypto/cc_priv.h>
#include "ccaes_internal.h"
#include "testmore.h"

struct vector {
    uint8_t in[32];
    uint8_t out[32];
};

static const struct vector vectors[] = {
    {
        {
            0xaf, 0xff, 0xbd, 0x2a, 0x2e, 0x6a, 0x33, 0xff, 0xe2, 0x1c, 0xf2, 0x9b, 0xad, 0x59, 0xed, 0x98,
            0xe8, 0x9c, 0x93, 0x9f, 0xf9, 0x6e, 0xeb, 0x49, 0x21, 0x5f, 0xbb, 0x54, 0x09, 0x1b, 0x12, 0x17,
        },
        {
            0x9f, 0xc5, 0xec, 0x56, 0x90, 0x50, 0x5f, 0xdd, 0xd9, 0xeb, 0x87, 0x18, 0x6d, 0x65, 0x58, 0x1b,
            0xae, 0xc9, 0x5c, 0xfe, 0x6f, 0xd4, 0x51, 0x40, 0x31, 0x94, 0xe8, 0xbc, 0x8c, 0x60, 0x13, 0x93,
        },
    },
    {
        {
            0x00, 0x68, 0x27, 0xaa, 0x5b, 0x0a, 0xc6, 0x87, 0xef, 0x79, 0xa1, 0x43, 0x57, 0xa9, 0xfb, 0xc3,
            0xb3, 0x82, 0x9a, 0x15, 0x1b, 0x86, 0x8d, 0xb7, 0xe7, 0xe1, 0xa4, 0xe7, 0x4d, 0xb0, 0x26, 0xc4,
        },
        {
            0x27, 0x9a, 0xee, 0x72, 0x4d, 0xda, 0xd9, 0x49, 0x5a, 0x4f, 0xe7, 0x1e, 0xf4, 0x4c, 0xa3, 0xd6,
            0x1a, 0x57, 0x84, 0xca, 0x2d, 0x14, 0x2c, 0x28, 0x43, 0x15, 0x96, 0x94, 0x7c, 0xbe, 0x85, 0xe1,
        },
    },
    {
        {
            0x85, 0xdd, 0x3c, 0xb2, 0x15, 0xd1, 0x34, 0xbf, 0x11, 0xa0, 0x8a, 0x6e, 0x1a, 0xf7, 0x04, 0xd5,
            0x4b, 0x2b, 0x67, 0xa1, 0x3d, 0xf2, 0xf0, 0xf3, 0x4d, 0x7a, 0x9d, 0x8e, 0x42, 0xbe, 0x5e, 0x53,
        },
        {
            0x6c, 0x8a, 0xbb, 0x3d, 0x0a, 0x11, 0x48, 0x25, 0x50, 0x32, 0x94, 0x4b, 0xab, 0x4c, 0x54, 0x23,
            0xa4, 0x8f, 0xab, 0x9a, 0xde, 0xd6, 0x25, 0x70, 0x91, 0x9b, 0x0f, 0x2a, 0xc2, 0xb1, 0xb8, 0x72,
        },
    },
    {
        {
            0x5e, 0xea, 0x42, 0xc1, 0x03, 0x8e, 0x07, 0x2e, 0x17, 0x19, 0x4e, 0x29, 0x1d, 0xe1, 0x8d, 0x98,
            0xbd, 0x9d, 0xae, 0xd4, 0x9a, 0x8d, 0xf2, 0xe2, 0x38, 0x45, 0x4e, 0x99, 0xf5, 0xb3, 0xd4, 0x45,
        },
        {
            0xbb, 0x08, 0x51, 0xe5, 0x46, 0x4d, 0xa6, 0x16, 0x0f, 0x12, 0x12, 0xa1, 0x93, 0xa3, 0xea, 0x0c,
            0x26, 0x90, 0x15, 0x5e, 0xea, 0xdc, 0x4e, 0x6b, 0xfb, 0x56, 0x98, 0x52, 0xf0, 0xd0, 0xec, 0x76,
        },
    },
    {
        {
            0x03, 0xb9, 0x11, 0x83, 0x7a, 0x76, 0x63, 0xd3, 0x2f, 0xa4, 0x87, 0x78, 0xf2, 0x6d, 0x5f, 0xd7,
            0x7d, 0xaf, 0x33, 0x3a, 0xef, 0x9d, 0xa5, 0x7c, 0xf8, 0x92, 0x95, 0x13, 0x39, 0x1f, 0x5c, 0x07,
        },
        {
            0xf7, 0x98, 0x2c, 0x63, 0x37, 0x9b, 0x07, 0x68, 0xea, 0x64, 0x2d, 0xe0, 0x1b, 0xc4, 0x9a, 0x3e,
            0x2e, 0xf8, 0x0e, 0xdf, 0x2b, 0xf8, 0x4b, 0x5a, 0xba, 0xb4, 0x60, 0xe8, 0xd5, 0xed, 0xc9, 0x5e,
        },
    },
    {
        {
            0xb9, 0x24, 0xd2, 0x06, 0xf1, 0xfc, 0x8b, 0xe1, 0x4c, 0x94, 0xb9, 0x8e, 0x90, 0xbc, 0x68, 0xac,
            0x2b, 0x01, 0xf6, 0xbf, 0xbc, 0xcf, 0x14, 0x15, 0x90, 0x94, 0x8e, 0x76, 0x35, 0x83, 0x24, 0x08,
        },
        {
            0xe6, 0x48, 0x37, 0x7d, 0xb6, 0x75, 0x10, 0x82, 0x95, 0xef, 0xf3, 0x0f, 0x41, 0x75, 0x99, 0xd3,
            0xfb, 0xd9, 0x93, 0x56, 0x64, 0x13, 0x54, 0x7c, 0xe2, 0xc2, 0x10, 0xba, 0x0e, 0xcc, 0x93, 0x52,
        },
    },
    {
        {
            0x9b, 0xed, 0xe4, 0x55, 0x7d, 0x9a, 0x42, 0x63, 0x26, 0x28, 0x6c, 0xf5, 0xd2, 0x76, 0xad, 0xde,
            0xc2, 0xec, 0xaa, 0x6b, 0xcf, 0xbc, 0xfb, 0x44, 0x29, 0x32, 0xda, 0x36, 0x58, 0xdf, 0xf3, 0xd1,
        },
        {
            0xc0, 0x86, 0xf1, 0xa1, 0x63, 0xea, 0x0e, 0xe2, 0xe6, 0xc8, 0x11, 0x81, 0x46, 0xcc, 0x17, 0xb3,
            0x31, 0x9e, 0x6b, 0xdc, 0x34, 0x07, 0xe5, 0xcd, 0x09, 0xe1, 0xe8, 0x53, 0x17, 0x9d, 0x8f, 0xbf,
        },
    },
    {
        {
            0xf6, 0xc8, 0x25, 0xa2, 0x84, 0x83, 0x0b, 0x24, 0x84, 0x8f, 0xc9, 0xb5, 0x30, 0x46, 0xd2, 0x35,
            0x67, 0xda, 0x09, 0x30, 0x0d, 0xe0, 0x44, 0x38, 0x34, 0xb1, 0xb3, 0x1a, 0xb4, 0x94, 0x13, 0x58,
        },
        {
            0x00, 0x8c, 0x53, 0xf3, 0x52, 0x7f, 0x94, 0xe8, 0x1c, 0x39, 0x00, 0x66, 0xfe, 0x5b, 0x37, 0x8e,
            0xbe, 0xd2, 0x06, 0xcb, 0xdb, 0x7a, 0x23, 0x1b, 0x2a, 0x50, 0x94, 0x0a, 0x88, 0x6f, 0xbc, 0x4e,
        },
    },
    {
        {
            0x00, 0xb2, 0xec, 0x29, 0xd6, 0xef, 0xaf, 0xa8, 0x3f, 0x27, 0xd6, 0x71, 0x69, 0x05, 0xe6, 0x4b,
            0x13, 0xb9, 0x9e, 0x3d, 0xef, 0x08, 0x88, 0x35, 0xb3, 0x31, 0x6a, 0xa9, 0x62, 0xa8, 0x1c, 0x5e,
        },
        {
            0x33, 0xec, 0x0c, 0x3e, 0x25, 0xb3, 0x11, 0xc1, 0x1f, 0xd0, 0xfc, 0x72, 0x9a, 0x84, 0xd7, 0x7e,
            0x78, 0xdb, 0xc0, 0x65, 0xdd, 0x7c, 0x32, 0x70, 0x5c, 0x5c, 0x0e, 0x56, 0x63, 0xff, 0x2b, 0x23,
        },
    },
    {
        {
            0x5d, 0xce, 0x9e, 0x28, 0x34, 0xc5, 0x64, 0x72, 0x0a, 0xc0, 0xba, 0xc0, 0x33, 0x41, 0xca, 0xcc,
            0xd7, 0x64, 0x7a, 0xe3, 0xcf, 0xd4, 0x7f, 0x0b, 0x0d, 0x45, 0xfb, 0x0b, 0xf8, 0x4c, 0x08, 0x15,
        },
        {
            0xf3, 0x67, 0x63, 0xfa, 0xae, 0xdd, 0xf6, 0xd8, 0xb6, 0x4a, 0xb9, 0x37, 0x3f, 0x32, 0x06, 0x28,
            0x55, 0xd7, 0xb7, 0x6e, 0xae, 0x24, 0xea, 0x44, 0xce, 0xdf, 0x1b, 0xfc, 0x1e, 0x4d, 0x07, 0x76,
        },
    },
    {
        {
            0xa0, 0xd3, 0x3b, 0xb5, 0xeb, 0x36, 0xaa, 0xa3, 0x35, 0x71, 0x36, 0x75, 0xb5, 0x90, 0x71, 0x90,
            0x2b, 0x37, 0x30, 0xb2, 0x9d, 0x55, 0x8d, 0x2f, 0x89, 0xc7, 0x6f, 0x37, 0x16, 0x63, 0xa9, 0xf2,
        },
        {
            0xdc, 0x43, 0x99, 0x81, 0xb7, 0xa9, 0xcc, 0x9a, 0x30, 0x05, 0x79, 0xe9, 0xe8, 0x04, 0xb7, 0x0f,
            0x8e, 0xbb, 0xe5, 0x1e, 0x1c, 0x31, 0x73, 0x5a, 0x46, 0xe6, 0x5b, 0x03, 0x67, 0xbb, 0xce, 0xfc,
        },
    },
    {
        {
            0x72, 0x98, 0x51, 0x4b, 0xc9, 0x5f, 0xd6, 0x86, 0x65, 0xfa, 0xe1, 0x4a, 0xb5, 0xd5, 0xef, 0xc6,
            0xea, 0xe1, 0xf8, 0x8e, 0xdc, 0x91, 0x9d, 0xdb, 0xe6, 0xc0, 0xaa, 0x15, 0x8c, 0xd0, 0x93, 0xd5,
        },
        {
            0x15, 0x01, 0x0c, 0x1e, 0x0d, 0x9a, 0xd2, 0xbc, 0x4f, 0x2c, 0x67, 0x6d, 0xf6, 0xd4, 0x7a, 0x37,
            0xbd, 0x6c, 0xa4, 0xfe, 0x9d, 0xff, 0x3d, 0x2b, 0xd2, 0x83, 0x60, 0x28, 0x23, 0xd4, 0x80, 0xbf,
        },
    },
    {
        {
            0xc7, 0x26, 0x80, 0xa2, 0x75, 0x74, 0x1a, 0x8e, 0x6c, 0x24, 0x88, 0x4f, 0xc2, 0xee, 0xf2, 0x8e,
            0xe9, 0x98, 0x48, 0x2b, 0x9d, 0x32, 0xb6, 0x0a, 0xb8, 0x60, 0xe0, 0x79, 0x71, 0xe8, 0xc6, 0xf5,
        },
        {
            0x4e, 0xd1, 0x6f, 0x78, 0x2e, 0x56, 0xcc, 0xd2, 0xa5, 0xe8, 0x6c, 0xb0, 0x30, 0x04, 0x92, 0x14,
            0x8a, 0x36, 0xc6, 0x2b, 0x3b, 0x40, 0x4e, 0x06, 0x27, 0x84, 0x2b, 0x6e, 0xac, 0xd7, 0xc4, 0x58,
        },
    },
    {
        {
            0xdf, 0x9b, 0x51, 0xbc, 0xd1, 0x10, 0x96, 0x09, 0xcf, 0x75, 0xd9, 0x39, 0x6e, 0xb0, 0x98, 0x58,
            0x8a, 0x34, 0xf4, 0x5d, 0x7c, 0xd1, 0x66, 0x69, 0x7a, 0xaf, 0x33, 0x33, 0xa1, 0xf5, 0x69, 0x4e,
        },
        {
            0x6d, 0x81, 0x9e, 0xe9, 0xcc, 0xfe, 0x36, 0x8c, 0xad, 0xb0, 0x7b, 0xd0, 0x57, 0x08, 0x7f, 0x2c,
            0xf3, 0xd2, 0xcf, 0xd4, 0x45, 0x0d, 0xed, 0xdb, 0xb6, 0x6a, 0x35, 0x5c, 0xdd, 0x3e, 0x15, 0x7b,
        },
    },
};

int ccaes_unwind_tests(TM_UNUSED int argc, TM_UNUSED char *const *argv)
{
    size_t nvectors = CC_ARRAY_LEN(vectors);

    plan_tests((int)nvectors * 2);

    for (unsigned i = 0; i < nvectors; i += 1) {
        uint8_t out[CCAES_KEY_SIZE_256];
        int ret = ccaes_unwind(sizeof(vectors[i].in), vectors[i].in, out);

        is(ret, CCERR_OK, "ccaes_unwind error");
        ok_memcmp(out, vectors[i].out, sizeof(out), "ccaes_unwind out");
    }

    return 1;
}
