# Copyright (c) (2011,2012,2013,2015,2016,2018,2019) Apple Inc. All rights reserved.
#
# corecrypto is licensed under Apple Inc.’s Internal Use License Agreement (which
# is contained in the License.txt file distributed with corecrypto) and only to 
# people who accept that license. IMPORTANT:  Any license rights granted to you by 
# Apple Inc. (if any) are limited to internal use within your organization only on 
# devices and computers you own or control, for the sole purpose of verifying the 
# security characteristics and correct functioning of the Apple Software.  You may 
# not, directly or indirectly, redistribute the Apple Software or any portions thereof.
#include <corecrypto/cc_config.h>

// This file was generated by MakeData.c. by edp for i386/x86_64 architecture.
/*

	the key expansion is still implemented in gen/aeskey.c
	The related tables _AESRcon and _AESInvMixColumnTable are not used and hence are commented out.

	The other 4 tables for aes_encrypt/aes_decrypt (atomic or cbc mode) are reduced to only the 1st table (Table 0).
	This is possible since in the arm architecture, the 2nd operand can be the rotated output of the barrel shiters.
	Since Tables 1/2/3, are rotated versions of Table 0, hence they are commented out.
	The assembly code in aes_cbc.s and EncryptDecrypt.s specifically use the barrel shiter rotation to provide
	the intended operands from Tables 1/2/3.

	1. removed commented out tables
	2. added compile switch to compile off unused tables

*/

#if CCAES_ARM_ASM

	CC_ASM_SECTION_CONST


// Round constants.
	.globl	_Rcon
	.private_extern	_Rcon
_Rcon:
	.byte	0x01
	.byte	0x02
	.byte	0x04
	.byte	0x08
	.byte	0x10
	.byte	0x20
	.byte	0x40
	.byte	0x80
	.byte	0x1b
	.byte	0x36


#if !defined(__arm64__)
// Tables for InvMixColumn.
	.globl	_AESInvMixColumnTable
	.private_extern	_AESInvMixColumnTable
	.p2align	2
_AESInvMixColumnTable:
	// Table 0.
	.long	0x00000000
	.long	0x0b0d090e
	.long	0x161a121c
	.long	0x1d171b12
	.long	0x2c342438
	.long	0x27392d36
	.long	0x3a2e3624
	.long	0x31233f2a
	.long	0x58684870
	.long	0x5365417e
	.long	0x4e725a6c
	.long	0x457f5362
	.long	0x745c6c48
	.long	0x7f516546
	.long	0x62467e54
	.long	0x694b775a
	.long	0xb0d090e0
	.long	0xbbdd99ee
	.long	0xa6ca82fc
	.long	0xadc78bf2
	.long	0x9ce4b4d8
	.long	0x97e9bdd6
	.long	0x8afea6c4
	.long	0x81f3afca
	.long	0xe8b8d890
	.long	0xe3b5d19e
	.long	0xfea2ca8c
	.long	0xf5afc382
	.long	0xc48cfca8
	.long	0xcf81f5a6
	.long	0xd296eeb4
	.long	0xd99be7ba
	.long	0x7bbb3bdb
	.long	0x70b632d5
	.long	0x6da129c7
	.long	0x66ac20c9
	.long	0x578f1fe3
	.long	0x5c8216ed
	.long	0x41950dff
	.long	0x4a9804f1
	.long	0x23d373ab
	.long	0x28de7aa5
	.long	0x35c961b7
	.long	0x3ec468b9
	.long	0x0fe75793
	.long	0x04ea5e9d
	.long	0x19fd458f
	.long	0x12f04c81
	.long	0xcb6bab3b
	.long	0xc066a235
	.long	0xdd71b927
	.long	0xd67cb029
	.long	0xe75f8f03
	.long	0xec52860d
	.long	0xf1459d1f
	.long	0xfa489411
	.long	0x9303e34b
	.long	0x980eea45
	.long	0x8519f157
	.long	0x8e14f859
	.long	0xbf37c773
	.long	0xb43ace7d
	.long	0xa92dd56f
	.long	0xa220dc61
	.long	0xf66d76ad
	.long	0xfd607fa3
	.long	0xe07764b1
	.long	0xeb7a6dbf
	.long	0xda595295
	.long	0xd1545b9b
	.long	0xcc434089
	.long	0xc74e4987
	.long	0xae053edd
	.long	0xa50837d3
	.long	0xb81f2cc1
	.long	0xb31225cf
	.long	0x82311ae5
	.long	0x893c13eb
	.long	0x942b08f9
	.long	0x9f2601f7
	.long	0x46bde64d
	.long	0x4db0ef43
	.long	0x50a7f451
	.long	0x5baafd5f
	.long	0x6a89c275
	.long	0x6184cb7b
	.long	0x7c93d069
	.long	0x779ed967
	.long	0x1ed5ae3d
	.long	0x15d8a733
	.long	0x08cfbc21
	.long	0x03c2b52f
	.long	0x32e18a05
	.long	0x39ec830b
	.long	0x24fb9819
	.long	0x2ff69117
	.long	0x8dd64d76
	.long	0x86db4478
	.long	0x9bcc5f6a
	.long	0x90c15664
	.long	0xa1e2694e
	.long	0xaaef6040
	.long	0xb7f87b52
	.long	0xbcf5725c
	.long	0xd5be0506
	.long	0xdeb30c08
	.long	0xc3a4171a
	.long	0xc8a91e14
	.long	0xf98a213e
	.long	0xf2872830
	.long	0xef903322
	.long	0xe49d3a2c
	.long	0x3d06dd96
	.long	0x360bd498
	.long	0x2b1ccf8a
	.long	0x2011c684
	.long	0x1132f9ae
	.long	0x1a3ff0a0
	.long	0x0728ebb2
	.long	0x0c25e2bc
	.long	0x656e95e6
	.long	0x6e639ce8
	.long	0x737487fa
	.long	0x78798ef4
	.long	0x495ab1de
	.long	0x4257b8d0
	.long	0x5f40a3c2
	.long	0x544daacc
	.long	0xf7daec41
	.long	0xfcd7e54f
	.long	0xe1c0fe5d
	.long	0xeacdf753
	.long	0xdbeec879
	.long	0xd0e3c177
	.long	0xcdf4da65
	.long	0xc6f9d36b
	.long	0xafb2a431
	.long	0xa4bfad3f
	.long	0xb9a8b62d
	.long	0xb2a5bf23
	.long	0x83868009
	.long	0x888b8907
	.long	0x959c9215
	.long	0x9e919b1b
	.long	0x470a7ca1
	.long	0x4c0775af
	.long	0x51106ebd
	.long	0x5a1d67b3
	.long	0x6b3e5899
	.long	0x60335197
	.long	0x7d244a85
	.long	0x7629438b
	.long	0x1f6234d1
	.long	0x146f3ddf
	.long	0x097826cd
	.long	0x02752fc3
	.long	0x335610e9
	.long	0x385b19e7
	.long	0x254c02f5
	.long	0x2e410bfb
	.long	0x8c61d79a
	.long	0x876cde94
	.long	0x9a7bc586
	.long	0x9176cc88
	.long	0xa055f3a2
	.long	0xab58faac
	.long	0xb64fe1be
	.long	0xbd42e8b0
	.long	0xd4099fea
	.long	0xdf0496e4
	.long	0xc2138df6
	.long	0xc91e84f8
	.long	0xf83dbbd2
	.long	0xf330b2dc
	.long	0xee27a9ce
	.long	0xe52aa0c0
	.long	0x3cb1477a
	.long	0x37bc4e74
	.long	0x2aab5566
	.long	0x21a65c68
	.long	0x10856342
	.long	0x1b886a4c
	.long	0x069f715e
	.long	0x0d927850
	.long	0x64d90f0a
	.long	0x6fd40604
	.long	0x72c31d16
	.long	0x79ce1418
	.long	0x48ed2b32
	.long	0x43e0223c
	.long	0x5ef7392e
	.long	0x55fa3020
	.long	0x01b79aec
	.long	0x0aba93e2
	.long	0x17ad88f0
	.long	0x1ca081fe
	.long	0x2d83bed4
	.long	0x268eb7da
	.long	0x3b99acc8
	.long	0x3094a5c6
	.long	0x59dfd29c
	.long	0x52d2db92
	.long	0x4fc5c080
	.long	0x44c8c98e
	.long	0x75ebf6a4
	.long	0x7ee6ffaa
	.long	0x63f1e4b8
	.long	0x68fcedb6
	.long	0xb1670a0c
	.long	0xba6a0302
	.long	0xa77d1810
	.long	0xac70111e
	.long	0x9d532e34
	.long	0x965e273a
	.long	0x8b493c28
	.long	0x80443526
	.long	0xe90f427c
	.long	0xe2024b72
	.long	0xff155060
	.long	0xf418596e
	.long	0xc53b6644
	.long	0xce366f4a
	.long	0xd3217458
	.long	0xd82c7d56
	.long	0x7a0ca137
	.long	0x7101a839
	.long	0x6c16b32b
	.long	0x671bba25
	.long	0x5638850f
	.long	0x5d358c01
	.long	0x40229713
	.long	0x4b2f9e1d
	.long	0x2264e947
	.long	0x2969e049
	.long	0x347efb5b
	.long	0x3f73f255
	.long	0x0e50cd7f
	.long	0x055dc471
	.long	0x184adf63
	.long	0x1347d66d
	.long	0xcadc31d7
	.long	0xc1d138d9
	.long	0xdcc623cb
	.long	0xd7cb2ac5
	.long	0xe6e815ef
	.long	0xede51ce1
	.long	0xf0f207f3
	.long	0xfbff0efd
	.long	0x92b479a7
	.long	0x99b970a9
	.long	0x84ae6bbb
	.long	0x8fa362b5
	.long	0xbe805d9f
	.long	0xb58d5491
	.long	0xa89a4f83
	.long	0xa397468d

// Tables for main encryption iterations.
	.globl	_AESEncryptTable
	.private_extern	_AESEncryptTable
	.p2align	2
_AESEncryptTable:
	// Table 0.
	.long	0xa56363c6
	.long	0x847c7cf8
	.long	0x997777ee
	.long	0x8d7b7bf6
	.long	0x0df2f2ff
	.long	0xbd6b6bd6
	.long	0xb16f6fde
	.long	0x54c5c591
	.long	0x50303060
	.long	0x03010102
	.long	0xa96767ce
	.long	0x7d2b2b56
	.long	0x19fefee7
	.long	0x62d7d7b5
	.long	0xe6abab4d
	.long	0x9a7676ec
	.long	0x45caca8f
	.long	0x9d82821f
	.long	0x40c9c989
	.long	0x877d7dfa
	.long	0x15fafaef
	.long	0xeb5959b2
	.long	0xc947478e
	.long	0x0bf0f0fb
	.long	0xecadad41
	.long	0x67d4d4b3
	.long	0xfda2a25f
	.long	0xeaafaf45
	.long	0xbf9c9c23
	.long	0xf7a4a453
	.long	0x967272e4
	.long	0x5bc0c09b
	.long	0xc2b7b775
	.long	0x1cfdfde1
	.long	0xae93933d
	.long	0x6a26264c
	.long	0x5a36366c
	.long	0x413f3f7e
	.long	0x02f7f7f5
	.long	0x4fcccc83
	.long	0x5c343468
	.long	0xf4a5a551
	.long	0x34e5e5d1
	.long	0x08f1f1f9
	.long	0x937171e2
	.long	0x73d8d8ab
	.long	0x53313162
	.long	0x3f15152a
	.long	0x0c040408
	.long	0x52c7c795
	.long	0x65232346
	.long	0x5ec3c39d
	.long	0x28181830
	.long	0xa1969637
	.long	0x0f05050a
	.long	0xb59a9a2f
	.long	0x0907070e
	.long	0x36121224
	.long	0x9b80801b
	.long	0x3de2e2df
	.long	0x26ebebcd
	.long	0x6927274e
	.long	0xcdb2b27f
	.long	0x9f7575ea
	.long	0x1b090912
	.long	0x9e83831d
	.long	0x742c2c58
	.long	0x2e1a1a34
	.long	0x2d1b1b36
	.long	0xb26e6edc
	.long	0xee5a5ab4
	.long	0xfba0a05b
	.long	0xf65252a4
	.long	0x4d3b3b76
	.long	0x61d6d6b7
	.long	0xceb3b37d
	.long	0x7b292952
	.long	0x3ee3e3dd
	.long	0x712f2f5e
	.long	0x97848413
	.long	0xf55353a6
	.long	0x68d1d1b9
	.long	0x00000000
	.long	0x2cededc1
	.long	0x60202040
	.long	0x1ffcfce3
	.long	0xc8b1b179
	.long	0xed5b5bb6
	.long	0xbe6a6ad4
	.long	0x46cbcb8d
	.long	0xd9bebe67
	.long	0x4b393972
	.long	0xde4a4a94
	.long	0xd44c4c98
	.long	0xe85858b0
	.long	0x4acfcf85
	.long	0x6bd0d0bb
	.long	0x2aefefc5
	.long	0xe5aaaa4f
	.long	0x16fbfbed
	.long	0xc5434386
	.long	0xd74d4d9a
	.long	0x55333366
	.long	0x94858511
	.long	0xcf45458a
	.long	0x10f9f9e9
	.long	0x06020204
	.long	0x817f7ffe
	.long	0xf05050a0
	.long	0x443c3c78
	.long	0xba9f9f25
	.long	0xe3a8a84b
	.long	0xf35151a2
	.long	0xfea3a35d
	.long	0xc0404080
	.long	0x8a8f8f05
	.long	0xad92923f
	.long	0xbc9d9d21
	.long	0x48383870
	.long	0x04f5f5f1
	.long	0xdfbcbc63
	.long	0xc1b6b677
	.long	0x75dadaaf
	.long	0x63212142
	.long	0x30101020
	.long	0x1affffe5
	.long	0x0ef3f3fd
	.long	0x6dd2d2bf
	.long	0x4ccdcd81
	.long	0x140c0c18
	.long	0x35131326
	.long	0x2fececc3
	.long	0xe15f5fbe
	.long	0xa2979735
	.long	0xcc444488
	.long	0x3917172e
	.long	0x57c4c493
	.long	0xf2a7a755
	.long	0x827e7efc
	.long	0x473d3d7a
	.long	0xac6464c8
	.long	0xe75d5dba
	.long	0x2b191932
	.long	0x957373e6
	.long	0xa06060c0
	.long	0x98818119
	.long	0xd14f4f9e
	.long	0x7fdcdca3
	.long	0x66222244
	.long	0x7e2a2a54
	.long	0xab90903b
	.long	0x8388880b
	.long	0xca46468c
	.long	0x29eeeec7
	.long	0xd3b8b86b
	.long	0x3c141428
	.long	0x79dedea7
	.long	0xe25e5ebc
	.long	0x1d0b0b16
	.long	0x76dbdbad
	.long	0x3be0e0db
	.long	0x56323264
	.long	0x4e3a3a74
	.long	0x1e0a0a14
	.long	0xdb494992
	.long	0x0a06060c
	.long	0x6c242448
	.long	0xe45c5cb8
	.long	0x5dc2c29f
	.long	0x6ed3d3bd
	.long	0xefacac43
	.long	0xa66262c4
	.long	0xa8919139
	.long	0xa4959531
	.long	0x37e4e4d3
	.long	0x8b7979f2
	.long	0x32e7e7d5
	.long	0x43c8c88b
	.long	0x5937376e
	.long	0xb76d6dda
	.long	0x8c8d8d01
	.long	0x64d5d5b1
	.long	0xd24e4e9c
	.long	0xe0a9a949
	.long	0xb46c6cd8
	.long	0xfa5656ac
	.long	0x07f4f4f3
	.long	0x25eaeacf
	.long	0xaf6565ca
	.long	0x8e7a7af4
	.long	0xe9aeae47
	.long	0x18080810
	.long	0xd5baba6f
	.long	0x887878f0
	.long	0x6f25254a
	.long	0x722e2e5c
	.long	0x241c1c38
	.long	0xf1a6a657
	.long	0xc7b4b473
	.long	0x51c6c697
	.long	0x23e8e8cb
	.long	0x7cdddda1
	.long	0x9c7474e8
	.long	0x211f1f3e
	.long	0xdd4b4b96
	.long	0xdcbdbd61
	.long	0x868b8b0d
	.long	0x858a8a0f
	.long	0x907070e0
	.long	0x423e3e7c
	.long	0xc4b5b571
	.long	0xaa6666cc
	.long	0xd8484890
	.long	0x05030306
	.long	0x01f6f6f7
	.long	0x120e0e1c
	.long	0xa36161c2
	.long	0x5f35356a
	.long	0xf95757ae
	.long	0xd0b9b969
	.long	0x91868617
	.long	0x58c1c199
	.long	0x271d1d3a
	.long	0xb99e9e27
	.long	0x38e1e1d9
	.long	0x13f8f8eb
	.long	0xb398982b
	.long	0x33111122
	.long	0xbb6969d2
	.long	0x70d9d9a9
	.long	0x898e8e07
	.long	0xa7949433
	.long	0xb69b9b2d
	.long	0x221e1e3c
	.long	0x92878715
	.long	0x20e9e9c9
	.long	0x49cece87
	.long	0xff5555aa
	.long	0x78282850
	.long	0x7adfdfa5
	.long	0x8f8c8c03
	.long	0xf8a1a159
	.long	0x80898909
	.long	0x170d0d1a
	.long	0xdabfbf65
	.long	0x31e6e6d7
	.long	0xc6424284
	.long	0xb86868d0
	.long	0xc3414182
	.long	0xb0999929
	.long	0x772d2d5a
	.long	0x110f0f1e
	.long	0xcbb0b07b
	.long	0xfc5454a8
	.long	0xd6bbbb6d
	.long	0x3a16162c


// Tables for main decryption iterations.
	.globl	_AESDecryptTable
	.private_extern	_AESDecryptTable
	.p2align	2
_AESDecryptTable:
	// Table 0.
	.long	0x50a7f451
	.long	0x5365417e
	.long	0xc3a4171a
	.long	0x965e273a
	.long	0xcb6bab3b
	.long	0xf1459d1f
	.long	0xab58faac
	.long	0x9303e34b
	.long	0x55fa3020
	.long	0xf66d76ad
	.long	0x9176cc88
	.long	0x254c02f5
	.long	0xfcd7e54f
	.long	0xd7cb2ac5
	.long	0x80443526
	.long	0x8fa362b5
	.long	0x495ab1de
	.long	0x671bba25
	.long	0x980eea45
	.long	0xe1c0fe5d
	.long	0x02752fc3
	.long	0x12f04c81
	.long	0xa397468d
	.long	0xc6f9d36b
	.long	0xe75f8f03
	.long	0x959c9215
	.long	0xeb7a6dbf
	.long	0xda595295
	.long	0x2d83bed4
	.long	0xd3217458
	.long	0x2969e049
	.long	0x44c8c98e
	.long	0x6a89c275
	.long	0x78798ef4
	.long	0x6b3e5899
	.long	0xdd71b927
	.long	0xb64fe1be
	.long	0x17ad88f0
	.long	0x66ac20c9
	.long	0xb43ace7d
	.long	0x184adf63
	.long	0x82311ae5
	.long	0x60335197
	.long	0x457f5362
	.long	0xe07764b1
	.long	0x84ae6bbb
	.long	0x1ca081fe
	.long	0x942b08f9
	.long	0x58684870
	.long	0x19fd458f
	.long	0x876cde94
	.long	0xb7f87b52
	.long	0x23d373ab
	.long	0xe2024b72
	.long	0x578f1fe3
	.long	0x2aab5566
	.long	0x0728ebb2
	.long	0x03c2b52f
	.long	0x9a7bc586
	.long	0xa50837d3
	.long	0xf2872830
	.long	0xb2a5bf23
	.long	0xba6a0302
	.long	0x5c8216ed
	.long	0x2b1ccf8a
	.long	0x92b479a7
	.long	0xf0f207f3
	.long	0xa1e2694e
	.long	0xcdf4da65
	.long	0xd5be0506
	.long	0x1f6234d1
	.long	0x8afea6c4
	.long	0x9d532e34
	.long	0xa055f3a2
	.long	0x32e18a05
	.long	0x75ebf6a4
	.long	0x39ec830b
	.long	0xaaef6040
	.long	0x069f715e
	.long	0x51106ebd
	.long	0xf98a213e
	.long	0x3d06dd96
	.long	0xae053edd
	.long	0x46bde64d
	.long	0xb58d5491
	.long	0x055dc471
	.long	0x6fd40604
	.long	0xff155060
	.long	0x24fb9819
	.long	0x97e9bdd6
	.long	0xcc434089
	.long	0x779ed967
	.long	0xbd42e8b0
	.long	0x888b8907
	.long	0x385b19e7
	.long	0xdbeec879
	.long	0x470a7ca1
	.long	0xe90f427c
	.long	0xc91e84f8
	.long	0x00000000
	.long	0x83868009
	.long	0x48ed2b32
	.long	0xac70111e
	.long	0x4e725a6c
	.long	0xfbff0efd
	.long	0x5638850f
	.long	0x1ed5ae3d
	.long	0x27392d36
	.long	0x64d90f0a
	.long	0x21a65c68
	.long	0xd1545b9b
	.long	0x3a2e3624
	.long	0xb1670a0c
	.long	0x0fe75793
	.long	0xd296eeb4
	.long	0x9e919b1b
	.long	0x4fc5c080
	.long	0xa220dc61
	.long	0x694b775a
	.long	0x161a121c
	.long	0x0aba93e2
	.long	0xe52aa0c0
	.long	0x43e0223c
	.long	0x1d171b12
	.long	0x0b0d090e
	.long	0xadc78bf2
	.long	0xb9a8b62d
	.long	0xc8a91e14
	.long	0x8519f157
	.long	0x4c0775af
	.long	0xbbdd99ee
	.long	0xfd607fa3
	.long	0x9f2601f7
	.long	0xbcf5725c
	.long	0xc53b6644
	.long	0x347efb5b
	.long	0x7629438b
	.long	0xdcc623cb
	.long	0x68fcedb6
	.long	0x63f1e4b8
	.long	0xcadc31d7
	.long	0x10856342
	.long	0x40229713
	.long	0x2011c684
	.long	0x7d244a85
	.long	0xf83dbbd2
	.long	0x1132f9ae
	.long	0x6da129c7
	.long	0x4b2f9e1d
	.long	0xf330b2dc
	.long	0xec52860d
	.long	0xd0e3c177
	.long	0x6c16b32b
	.long	0x99b970a9
	.long	0xfa489411
	.long	0x2264e947
	.long	0xc48cfca8
	.long	0x1a3ff0a0
	.long	0xd82c7d56
	.long	0xef903322
	.long	0xc74e4987
	.long	0xc1d138d9
	.long	0xfea2ca8c
	.long	0x360bd498
	.long	0xcf81f5a6
	.long	0x28de7aa5
	.long	0x268eb7da
	.long	0xa4bfad3f
	.long	0xe49d3a2c
	.long	0x0d927850
	.long	0x9bcc5f6a
	.long	0x62467e54
	.long	0xc2138df6
	.long	0xe8b8d890
	.long	0x5ef7392e
	.long	0xf5afc382
	.long	0xbe805d9f
	.long	0x7c93d069
	.long	0xa92dd56f
	.long	0xb31225cf
	.long	0x3b99acc8
	.long	0xa77d1810
	.long	0x6e639ce8
	.long	0x7bbb3bdb
	.long	0x097826cd
	.long	0xf418596e
	.long	0x01b79aec
	.long	0xa89a4f83
	.long	0x656e95e6
	.long	0x7ee6ffaa
	.long	0x08cfbc21
	.long	0xe6e815ef
	.long	0xd99be7ba
	.long	0xce366f4a
	.long	0xd4099fea
	.long	0xd67cb029
	.long	0xafb2a431
	.long	0x31233f2a
	.long	0x3094a5c6
	.long	0xc066a235
	.long	0x37bc4e74
	.long	0xa6ca82fc
	.long	0xb0d090e0
	.long	0x15d8a733
	.long	0x4a9804f1
	.long	0xf7daec41
	.long	0x0e50cd7f
	.long	0x2ff69117
	.long	0x8dd64d76
	.long	0x4db0ef43
	.long	0x544daacc
	.long	0xdf0496e4
	.long	0xe3b5d19e
	.long	0x1b886a4c
	.long	0xb81f2cc1
	.long	0x7f516546
	.long	0x04ea5e9d
	.long	0x5d358c01
	.long	0x737487fa
	.long	0x2e410bfb
	.long	0x5a1d67b3
	.long	0x52d2db92
	.long	0x335610e9
	.long	0x1347d66d
	.long	0x8c61d79a
	.long	0x7a0ca137
	.long	0x8e14f859
	.long	0x893c13eb
	.long	0xee27a9ce
	.long	0x35c961b7
	.long	0xede51ce1
	.long	0x3cb1477a
	.long	0x59dfd29c
	.long	0x3f73f255
	.long	0x79ce1418
	.long	0xbf37c773
	.long	0xeacdf753
	.long	0x5baafd5f
	.long	0x146f3ddf
	.long	0x86db4478
	.long	0x81f3afca
	.long	0x3ec468b9
	.long	0x2c342438
	.long	0x5f40a3c2
	.long	0x72c31d16
	.long	0x0c25e2bc
	.long	0x8b493c28
	.long	0x41950dff
	.long	0x7101a839
	.long	0xdeb30c08
	.long	0x9ce4b4d8
	.long	0x90c15664
	.long	0x6184cb7b
	.long	0x70b632d5
	.long	0x745c6c48
	.long	0x4257b8d0

#endif		// __arm64__


// SubBytes embedded in words tables.
	.globl	_AESSubBytesWordTable
	.private_extern	_AESSubBytesWordTable
	.p2align	2
_AESSubBytesWordTable:
	// Table 0.
	// this table is updated to normal byte table as it is only needed in the aes key expansion code
	.byte	0x63
	.byte	0x7c
	.byte	0x77
	.byte	0x7b
	.byte	0xf2
	.byte	0x6b
	.byte	0x6f
	.byte	0xc5
	.byte	0x30
	.byte	0x01
	.byte	0x67
	.byte	0x2b
	.byte	0xfe
	.byte	0xd7
	.byte	0xab
	.byte	0x76
	.byte	0xca
	.byte	0x82
	.byte	0xc9
	.byte	0x7d
	.byte	0xfa
	.byte	0x59
	.byte	0x47
	.byte	0xf0
	.byte	0xad
	.byte	0xd4
	.byte	0xa2
	.byte	0xaf
	.byte	0x9c
	.byte	0xa4
	.byte	0x72
	.byte	0xc0
	.byte	0xb7
	.byte	0xfd
	.byte	0x93
	.byte	0x26
	.byte	0x36
	.byte	0x3f
	.byte	0xf7
	.byte	0xcc
	.byte	0x34
	.byte	0xa5
	.byte	0xe5
	.byte	0xf1
	.byte	0x71
	.byte	0xd8
	.byte	0x31
	.byte	0x15
	.byte	0x04
	.byte	0xc7
	.byte	0x23
	.byte	0xc3
	.byte	0x18
	.byte	0x96
	.byte	0x05
	.byte	0x9a
	.byte	0x07
	.byte	0x12
	.byte	0x80
	.byte	0xe2
	.byte	0xeb
	.byte	0x27
	.byte	0xb2
	.byte	0x75
	.byte	0x09
	.byte	0x83
	.byte	0x2c
	.byte	0x1a
	.byte	0x1b
	.byte	0x6e
	.byte	0x5a
	.byte	0xa0
	.byte	0x52
	.byte	0x3b
	.byte	0xd6
	.byte	0xb3
	.byte	0x29
	.byte	0xe3
	.byte	0x2f
	.byte	0x84
	.byte	0x53
	.byte	0xd1
	.byte	0x00
	.byte	0xed
	.byte	0x20
	.byte	0xfc
	.byte	0xb1
	.byte	0x5b
	.byte	0x6a
	.byte	0xcb
	.byte	0xbe
	.byte	0x39
	.byte	0x4a
	.byte	0x4c
	.byte	0x58
	.byte	0xcf
	.byte	0xd0
	.byte	0xef
	.byte	0xaa
	.byte	0xfb
	.byte	0x43
	.byte	0x4d
	.byte	0x33
	.byte	0x85
	.byte	0x45
	.byte	0xf9
	.byte	0x02
	.byte	0x7f
	.byte	0x50
	.byte	0x3c
	.byte	0x9f
	.byte	0xa8
	.byte	0x51
	.byte	0xa3
	.byte	0x40
	.byte	0x8f
	.byte	0x92
	.byte	0x9d
	.byte	0x38
	.byte	0xf5
	.byte	0xbc
	.byte	0xb6
	.byte	0xda
	.byte	0x21
	.byte	0x10
	.byte	0xff
	.byte	0xf3
	.byte	0xd2
	.byte	0xcd
	.byte	0x0c
	.byte	0x13
	.byte	0xec
	.byte	0x5f
	.byte	0x97
	.byte	0x44
	.byte	0x17
	.byte	0xc4
	.byte	0xa7
	.byte	0x7e
	.byte	0x3d
	.byte	0x64
	.byte	0x5d
	.byte	0x19
	.byte	0x73
	.byte	0x60
	.byte	0x81
	.byte	0x4f
	.byte	0xdc
	.byte	0x22
	.byte	0x2a
	.byte	0x90
	.byte	0x88
	.byte	0x46
	.byte	0xee
	.byte	0xb8
	.byte	0x14
	.byte	0xde
	.byte	0x5e
	.byte	0x0b
	.byte	0xdb
	.byte	0xe0
	.byte	0x32
	.byte	0x3a
	.byte	0x0a
	.byte	0x49
	.byte	0x06
	.byte	0x24
	.byte	0x5c
	.byte	0xc2
	.byte	0xd3
	.byte	0xac
	.byte	0x62
	.byte	0x91
	.byte	0x95
	.byte	0xe4
	.byte	0x79
	.byte	0xe7
	.byte	0xc8
	.byte	0x37
	.byte	0x6d
	.byte	0x8d
	.byte	0xd5
	.byte	0x4e
	.byte	0xa9
	.byte	0x6c
	.byte	0x56
	.byte	0xf4
	.byte	0xea
	.byte	0x65
	.byte	0x7a
	.byte	0xae
	.byte	0x08
	.byte	0xba
	.byte	0x78
	.byte	0x25
	.byte	0x2e
	.byte	0x1c
	.byte	0xa6
	.byte	0xb4
	.byte	0xc6
	.byte	0xe8
	.byte	0xdd
	.byte	0x74
	.byte	0x1f
	.byte	0x4b
	.byte	0xbd
	.byte	0x8b
	.byte	0x8a
	.byte	0x70
	.byte	0x3e
	.byte	0xb5
	.byte	0x66
	.byte	0x48
	.byte	0x03
	.byte	0xf6
	.byte	0x0e
	.byte	0x61
	.byte	0x35
	.byte	0x57
	.byte	0xb9
	.byte	0x86
	.byte	0xc1
	.byte	0x1d
	.byte	0x9e
	.byte	0xe1
	.byte	0xf8
	.byte	0x98
	.byte	0x11
	.byte	0x69
	.byte	0xd9
	.byte	0x8e
	.byte	0x94
	.byte	0x9b
	.byte	0x1e
	.byte	0x87
	.byte	0xe9
	.byte	0xce
	.byte	0x55
	.byte	0x28
	.byte	0xdf
	.byte	0x8c
	.byte	0xa1
	.byte	0x89
	.byte	0x0d
	.byte	0xbf
	.byte	0xe6
	.byte	0x42
	.byte	0x68
	.byte	0x41
	.byte	0x99
	.byte	0x2d
	.byte	0x0f
	.byte	0xb0
	.byte	0x54
	.byte	0xbb
	.byte	0x16


#if !defined(__arm64__)
// InvSubBytes embedded in words tables.
	.globl	_AESInvSubBytesWordTable
	.private_extern	_AESInvSubBytesWordTable
	.p2align	2
_AESInvSubBytesWordTable:
	// Table 0.
	.byte	0x52
	.byte	0x09
	.byte	0x6a
	.byte	0xd5
	.byte	0x30
	.byte	0x36
	.byte	0xa5
	.byte	0x38
	.byte	0xbf
	.byte	0x40
	.byte	0xa3
	.byte	0x9e
	.byte	0x81
	.byte	0xf3
	.byte	0xd7
	.byte	0xfb
	.byte	0x7c
	.byte	0xe3
	.byte	0x39
	.byte	0x82
	.byte	0x9b
	.byte	0x2f
	.byte	0xff
	.byte	0x87
	.byte	0x34
	.byte	0x8e
	.byte	0x43
	.byte	0x44
	.byte	0xc4
	.byte	0xde
	.byte	0xe9
	.byte	0xcb
	.byte	0x54
	.byte	0x7b
	.byte	0x94
	.byte	0x32
	.byte	0xa6
	.byte	0xc2
	.byte	0x23
	.byte	0x3d
	.byte	0xee
	.byte	0x4c
	.byte	0x95
	.byte	0x0b
	.byte	0x42
	.byte	0xfa
	.byte	0xc3
	.byte	0x4e
	.byte	0x08
	.byte	0x2e
	.byte	0xa1
	.byte	0x66
	.byte	0x28
	.byte	0xd9
	.byte	0x24
	.byte	0xb2
	.byte	0x76
	.byte	0x5b
	.byte	0xa2
	.byte	0x49
	.byte	0x6d
	.byte	0x8b
	.byte	0xd1
	.byte	0x25
	.byte	0x72
	.byte	0xf8
	.byte	0xf6
	.byte	0x64
	.byte	0x86
	.byte	0x68
	.byte	0x98
	.byte	0x16
	.byte	0xd4
	.byte	0xa4
	.byte	0x5c
	.byte	0xcc
	.byte	0x5d
	.byte	0x65
	.byte	0xb6
	.byte	0x92
	.byte	0x6c
	.byte	0x70
	.byte	0x48
	.byte	0x50
	.byte	0xfd
	.byte	0xed
	.byte	0xb9
	.byte	0xda
	.byte	0x5e
	.byte	0x15
	.byte	0x46
	.byte	0x57
	.byte	0xa7
	.byte	0x8d
	.byte	0x9d
	.byte	0x84
	.byte	0x90
	.byte	0xd8
	.byte	0xab
	.byte	0x00
	.byte	0x8c
	.byte	0xbc
	.byte	0xd3
	.byte	0x0a
	.byte	0xf7
	.byte	0xe4
	.byte	0x58
	.byte	0x05
	.byte	0xb8
	.byte	0xb3
	.byte	0x45
	.byte	0x06
	.byte	0xd0
	.byte	0x2c
	.byte	0x1e
	.byte	0x8f
	.byte	0xca
	.byte	0x3f
	.byte	0x0f
	.byte	0x02
	.byte	0xc1
	.byte	0xaf
	.byte	0xbd
	.byte	0x03
	.byte	0x01
	.byte	0x13
	.byte	0x8a
	.byte	0x6b
	.byte	0x3a
	.byte	0x91
	.byte	0x11
	.byte	0x41
	.byte	0x4f
	.byte	0x67
	.byte	0xdc
	.byte	0xea
	.byte	0x97
	.byte	0xf2
	.byte	0xcf
	.byte	0xce
	.byte	0xf0
	.byte	0xb4
	.byte	0xe6
	.byte	0x73
	.byte	0x96
	.byte	0xac
	.byte	0x74
	.byte	0x22
	.byte	0xe7
	.byte	0xad
	.byte	0x35
	.byte	0x85
	.byte	0xe2
	.byte	0xf9
	.byte	0x37
	.byte	0xe8
	.byte	0x1c
	.byte	0x75
	.byte	0xdf
	.byte	0x6e
	.byte	0x47
	.byte	0xf1
	.byte	0x1a
	.byte	0x71
	.byte	0x1d
	.byte	0x29
	.byte	0xc5
	.byte	0x89
	.byte	0x6f
	.byte	0xb7
	.byte	0x62
	.byte	0x0e
	.byte	0xaa
	.byte	0x18
	.byte	0xbe
	.byte	0x1b
	.byte	0xfc
	.byte	0x56
	.byte	0x3e
	.byte	0x4b
	.byte	0xc6
	.byte	0xd2
	.byte	0x79
	.byte	0x20
	.byte	0x9a
	.byte	0xdb
	.byte	0xc0
	.byte	0xfe
	.byte	0x78
	.byte	0xcd
	.byte	0x5a
	.byte	0xf4
	.byte	0x1f
	.byte	0xdd
	.byte	0xa8
	.byte	0x33
	.byte	0x88
	.byte	0x07
	.byte	0xc7
	.byte	0x31
	.byte	0xb1
	.byte	0x12
	.byte	0x10
	.byte	0x59
	.byte	0x27
	.byte	0x80
	.byte	0xec
	.byte	0x5f
	.byte	0x60
	.byte	0x51
	.byte	0x7f
	.byte	0xa9
	.byte	0x19
	.byte	0xb5
	.byte	0x4a
	.byte	0x0d
	.byte	0x2d
	.byte	0xe5
	.byte	0x7a
	.byte	0x9f
	.byte	0x93
	.byte	0xc9
	.byte	0x9c
	.byte	0xef
	.byte	0xa0
	.byte	0xe0
	.byte	0x3b
	.byte	0x4d
	.byte	0xae
	.byte	0x2a
	.byte	0xf5
	.byte	0xb0
	.byte	0xc8
	.byte	0xeb
	.byte	0xbb
	.byte	0x3c
	.byte	0x83
	.byte	0x53
	.byte	0x99
	.byte	0x61
	.byte	0x17
	.byte	0x2b
	.byte	0x04
	.byte	0x7e
	.byte	0xba
	.byte	0x77
	.byte	0xd6
	.byte	0x26
	.byte	0xe1
	.byte	0x69
	.byte	0x14
	.byte	0x63
	.byte	0x55
	.byte	0x21
	.byte	0x0c
	.byte	0x7d

#endif		// !defined(__arm64__)

#endif /* CCAES_ARM_ASM */

